#!/usr/bin/env perl
use strict;
use constant RBLOCK_DIR => 'rblock';



unless(scalar @ARGV){
	print "filename required\n";
	exit();
}

my $filename = shift @ARGV;
my $tmp_filename = $filename;

$tmp_filename =~ s/\.java$/__tmp.java/;


sub tr_class{
	my $name = shift;
	my @pkg_list = split(/\./, $name);
	my $class = pop(@pkg_list);
	($class =~ s/^([a-z])//) and ($class = uc($1).$class);
	push(@pkg_list, $class);
	
	return join('.', @pkg_list);
}

sub mk_pkg{
	my $name = shift;
	my @pkg_list = split(/\./, $name);
	pop(@pkg_list);

	my $path = RBLOCK_DIR;
	foreach my $pkg (@pkg_list){
		$path = $path .'/'. $pkg;
		(-d $path) or (mkdir($path) or die 'error: '.$path);
	}
	return;
}

sub f{
	my $type = shift;
	my $src = shift;

	if($$src =~ s/^((?:(?:[\\].|[^\}\"\\])*?(?:\"(?:[\\].|[^\"\\])*?\"))*?(?:[\\].|[^\}\"\\])*?\{)//s){
		my $h = $1;
		return $h.f($src);
	}elsif($$src =~ s/^((?:(?:[\\].|[^\{\"\\])*?(?:\"(?:[\\].|[^\"\\])*?\"))*?(?:[\\].|[^\{\"\\])*?\})//s){
		my $h = $1;
		return $h;
	}else{
		return "";
	}
}

open(IN, "<", $filename) or die "Can't open $filename";
my $src = join("", <IN>);
close(IN);

my $class_table = {};

my $result = [];
while ($src =~ s/^((?:(?:[\\].|[^\"\\])*?(?:\"(?:[\\].|[^\"\\])*?\"))*?(?:[\\].|[^\"\\])*?)\s*?([a-zA-Z][a-zA-Z0-9]*(?:\.[a-zA-Z][a-zA-Z0-9]*)*|)\s*\{\s*\|\s*([a-zA-Z][a-zA-Z0-9]*)\s*\|\s*//so) {
	my $plain = $1;
	my $return_type = (length $2) ? $2 : 'java.lang.Object';
	my $arg_name = $3;
	my $class = tr_class((length $2) ? $2 : 'Object');

	unless(exists $class_table->{$class}){
		open(OUT, '>', sprintf('%s/%s.java', RBLOCK_DIR, $class)) or die 'error';
		print OUT "package rblock;\n";
		print OUT sprintf("public interface %s {\n", $class);
		print OUT sprintf("\tpublic %s call(java.lang.Object argument);\n", $return_type);
		print OUT "}";
		close(OUT);
	}

	my $content = f($return_type, \$src);

	push(@$result, $plain);
	push(@$result, sprintf(' new rblock.%s (){ ', $class));
	push(@$result, sprintf("public %s call(Object %s) { \n", $return_type, $arg_name));
	push(@$result, $content);
	push(@$result, ' } ');
}
push(@$result, $src);


#print join("", @$result);
#exit;

open(OUT, ">", $tmp_filename) or die "Can't open $tmp_filename";
print OUT join("", @$result);
close(OUT);
system("javac $tmp_filename");



